
## TODO: Remove once Alba's fun is merged in CSTools
source("tools/tmp/CST_Calibration.R")

## Entry params data and recipe?
calibrate_datasets <- function(data, recipe) {
  # Function that calibrates the hindcast using the method stated in the
  # recipe. If the forecast is not null, it calibrates it as well.
  #
  # data: list of s2dv_cube objects containing the hcst, obs and fcst.
  # recipe: object obtained when passing the .yml recipe file to read_yaml()
  
  method <- tolower(recipe$Analysis$Workflow$Calibration$method)

  if (method == "raw") {
    warning("The Calibration module has been called, but the calibration ",
	    "method in the recipe is 'raw'. The hcst and fcst will not be ",
	    "calibrated.")
    fcst_calibrated <- data$fcst
    hcst_calibrated <- data$hcst
    CALIB_MSG <- "##### NO CALIBRATION PERFORMED #####"

  } else {
    # Calibration function params
    mm <- recipe$Analysis$Datasets$Multimodel
    ncores <- 4
    na.rm <- T
    CALIB_MSG <- "##### CALIBRATION COMPLETE #####"
    # Replicate observation array for the multi-model case
    if (mm) {
      obs.mm <- obs$data
      for(dat in 1:(dim(data$hcst$data)['dat'][[1]]-1)) {
        obs.mm <- abind(obs.mm, data$obs$data,
  		      along=which(names(dim(data$obs$data)) == 'dat'))
      }
      names(dim(obs.mm)) <- names(dim(obs$data))
      data$obs$data <- obs.mm
      remove(obs.mm)
    }
  
    if (recipe$Analysis$Variables$freq == "monthly_mean") {
    
      CST_CALIB_METHODS <- c("bias", "evmos", "mse_min", "crps_min", "rpc-based")
      ## TODO: implement other calibration methods
      ## TODO: Restructure the code?
      if (!(method %in% CST_CALIB_METHODS)) {
        stop("Calibration method in the recipe is not available for monthly",
  	   " data.")
      } else {
        ## Alba's version of CST_Calibration (pending merge) is being used
        # Calibrate the hindcast
        hcst_calibrated <- CST_Calibration(data$hcst, data$obs,
                                           cal.method = method,
                                           eval.method = "leave-one-out",
                                           multi.model = mm,
                                           na.fill = TRUE,
                                           na.rm = na.rm,
                                           apply_to = NULL,
                                           alpha = NULL,
                                           memb_dim = "ensemble",
                                           sdate_dim = "syear",
                                           ncores = ncores)
        if (!is.null(data$fcst)) {
          # Calibrate the forecast
          fcst_calibrated <- CST_Calibration(data$hcst, data$obs, data$fcst,
                                             cal.method = method,
                                             eval.method = "leave-one-out",
                                             multi.model = mm,
                                             na.fill = TRUE,
                                             na.rm = na.rm,
                                             apply_to = NULL,
                                             alpha = NULL,
                                             memb_dim = "ensemble",
                                             sdate_dim = "syear",
                                             ncores = ncores)
        } else {
          fcst_calibrated <- NULL
        }
      } 
    } else if (recipe$Analysis$Variables$freq == "daily_mean") {
      # Daily data calibration using Quantile Mapping
      if (!(method %in% c("qmap"))) {
        stop("Calibration method in the recipe is not available at daily ",
  	   "frequency. Only quantile mapping 'qmap' is implemented.")
      }
      # Calibrate the hindcast
      hcst_calibrated <- CST_QuantileMapping(data$hcst, data$obs,
        			             exp_cor = NULL,
        			             sample_dims = c("syear",
        						     "time",
        						     "ensemble"),
        				     sample_length = NULL,
        				     method = "QUANT",
					     wet.day = FALSE,
        				     ncores = ncores,
        				     na.rm = na.rm)
  
      if (!is.null(data$fcst)) {
        # Calibrate the forecast
        fcst_calibrated <- CST_QuantileMapping(data$hcst, data$obs,
        				       exp_cor = data$fcst,
        				       sample_dims = c("syear",
        						       "time",
        						       "ensemble"),
        				       sample_length = NULL,
        				       method = "QUANT",
					       wet.day = FALSE,
        				       ncores = ncores,
        				       na.rm = na.rm)
      } else {
        fcst_calibrated <- NULL
      }
    }
  }
print(CALIB_MSG)
  ## TODO: Return observations too?
  return(list(hcst = hcst_calibrated, fcst = fcst_calibrated))
}
