# Loading module:
# 1. archive.yml
# 2. recipe.yml
# 3. Load_decadal.R (V)
#setwd('/esarchive/scratch/aho/git/auto-s2s/')

## TODO: remove paths to personal scratchs 
source("/esarchive/scratch/vagudets/repos/csoperational/R/get_regrid_params.R")
#source("/esarchive/scratch/vagudets/repos/cstools/R/s2dv_cube.R")
# Load required libraries/funs
source("modules/Loading/get_daily_time_ind.R")
source("modules/Loading/dates2load.R")
source("modules/Loading/check_latlon.R")
source("tools/libs.R")

#====================================================================

# recipe_file <- "modules/Loading/testing_recipes/recipe_decadal.yml"
# recipe_file <- "modules/Loading/testing_recipes/recipe_decadal_daily.yml"

load_datasets <- function(recipe_file) {

  recipe <- read_yaml(recipe_file)
  recipe$filename <- recipe_file 
  archive <- read_yaml(paste0(recipe$Run$code_dir, "conf/archive_decadal.yml"))$archive

  #-------------------------
  # Read from recipe:
  #-------------------------
  exp.name <- recipe$Analysis$Datasets$System$name #'HadGEM3'
  ref.name <- recipe$Analysis$Datasets$Reference$name #'era5'
  member <- strsplit(recipe$Analysis$Datasets$System$member, ',')[[1]] #c("r1i1p1f2", "r2i1p1f2")
  variable <- recipe$Analysis$Variables$name #'tas'
  store.freq <- recipe$Analysis$Variables$freq #monthly_mean
  lats.min <- as.numeric(recipe$Analysis$Region$latmin) #0
  lats.max <- as.numeric(recipe$Analysis$Region$latmax) #10
  lons.min <- as.numeric(recipe$Analysis$Region$lonmin) #0
  lons.max <- as.numeric(recipe$Analysis$Region$lonmax) #10

  # change to: sdates <- dates2load(recipe, logger)
  sdates_hcst <- as.numeric(recipe$Analysis$Time$hcst_start):as.numeric(recipe$Analysis$Time$hcst_end) #1960:2015
  sdates_fcst <- recipe$Analysis$Time$fcst
 
  #NOTE: time in recipe starts from 0, but in Start() it starts from 1 
  if (store.freq == "monthly_mean") {
    time_ind <- (as.numeric(recipe$Analysis$Time$ftime_min):as.numeric(recipe$Analysis$Time$ftime_max)) + 1

  } else if (store.freq == "daily_mean") {
    time_ind <- get_daily_time_ind(ftimemin = as.numeric(recipe$Analysis$Time$ftime_min),
                                   ftimemax = as.numeric(recipe$Analysis$Time$ftime_max),
                                   initial_month = archive$System[[exp.name]]$initial_month,
                                   sdates = sdates_hcst,
                                   calendar = archive$System[[exp.name]][[store.freq]]$calendar)
  }

#NOTE: May be used in the future
#  season <- recipe$Analysis$Time$season

  #-------------------------
  # Read from archive:
  #-------------------------
  if (store.freq == "monthly_mean") {
    table <- archive$System[[exp.name]][[store.freq]]$table[[variable]]  #'Amon'
  } else {
    table <- 'day'
  }
  grid <- archive$System[[exp.name]][[store.freq]]$grid[[variable]]
  version <- archive$System[[exp.name]][[store.freq]]$version[[variable]]
  if (identical(member, 'all')) {
    member <- strsplit(archive$System[[exp.name]]$member, ',')[[1]]
  }

  #-------------------------
  # derived from above:
  #-------------------------
  # Check lat and lon and decide CircularSort
   circularsort <- check_latlon(latmin = lats.min, latmax = lats.max, lonmin = lons.min, lonmax = lons.max)

  # generate transform params for system and ref
  regrid_params <- get_regrid_params(recipe, archive)

  # Only if the time length in each chunk may differ that we need largest_dims_length to be TRUE. Otherwise, set FALSE to increase efficiency.
  need_largest_dims_length <- ifelse(exp.name == 'EC-Earth3-i2', TRUE, FALSE) 


  #-------------------------------------------
  # Step 1: Load the hcst
  #-------------------------------------------
  #monthly and daily
  hcst.path <- file.path(archive$src, archive$System[[exp.name]]$src$hcst,
                         '$ensemble$', table, '$var$', grid, version)
  hcst.files <- paste0('$var$_', table, '_*_*_s$syear$-$ensemble$_', grid, '_$chunk$.nc')

  Start_hcst_arg_list <- list(
    dat = file.path(hcst.path, hcst.files),
    var = variable,
    ensemble = member,
    syear = paste0(sdates_hcst),
    chunk = 'all',
    chunk_depends = 'syear',
    time = indices(time_ind),
    time_across = 'chunk',
    merge_across_dims = TRUE,
    largest_dims_length = need_largest_dims_length,
    latitude = values(list(lats.min, lats.max)),
    latitude_reorder = Sort(decreasing = TRUE),
    longitude = values(list(lons.min, lons.max)),
    longitude_reorder = circularsort,
    transform = regrid_params$fcst.transform,
    transform_extra_cells = 2,
    transform_params = list(grid = regrid_params$fcst.gridtype,
                            method = regrid_params$fcst.gridmethod),
    transform_vars = c('latitude', 'longitude'),
    synonims = list(longitude = c('lon', 'longitude'),
                    latitude = c('lat', 'latitude')),
    return_vars = list(latitude = NULL, longitude = NULL,
                       time = c('syear', 'chunk')),
    retrieve = T)

  hcst <- do.call(Start, Start_hcst_arg_list)

  tmp_time_attr <- attr(hcst, 'Variables')$common$time


  # change syear to c(sday, sweek, syear)
  dim(hcst) <- c(dim(hcst)[1:3], sday = 1, sweek = 1, dim(hcst)[4:7])
  if (!identical(dim(tmp_time_attr), dim(hcst)[6:7])) {
    stop("hcst has problem in matching data and time attr dimension.")
  }
  dim(attr(hcst, 'Variables')$common$time) <- c(sday = 1, sweek = 1, dim(hcst)[6:7])

  # Change class from startR_array to s2dv_cube
  suppressWarnings(
  hcst <- as.s2dv_cube(hcst)
  )

#-------------------------------------------
# Step 2: Load the fcst
#-------------------------------------------
  if (!is.null(recipe$Analysis$Time$fcst)) {

  # Define path (monthly and daily)
  multi_path <- FALSE
  if (is.null(archive$System[[exp.name]]$src$first_dcppB_syear)) { # only dcppA
    fcst.path <- file.path(archive$src, archive$System[[exp.name]]$src$hcst,
                           '$ensemble$', table, '$var$', grid, version)
    fcst.files <- paste0('$var$_', table, '_*_dcppA-hindcast_s$syear$-$ensemble$_', grid, '_$chunk$.nc')

  } else {
    if (all(sdates_fcst >= archive$System[[exp.name]]$src$first_dcppB_syear)) { # only dcppB
      fcst.path <- file.path(archive$src, archive$System[[exp.name]]$src$fcst,
                             '$ensemble$', table, '$var$', grid, version)
      fcst.files <- paste0('$var$_', table, '_*_dcppB-forecast_s$syear$-$ensemble$_', grid, '_$chunk$.nc')

    } else {
      # Create one path for each sdate
      #TODO: When *_depends = 'syear' can be more than one, use one path with $dcpp$
      multi_path <- TRUE
      path_list <- vector('list', length = length(sdates_fcst))
      for (i_sdate in 1:length(sdates_fcst)) {
        if (sdates_fcst[i_sdate] >= archive$System[[exp.name]]$src$first_dcppB_syear) {
          path_list[[i_sdate]] <- 
            list(path = file.path(archive$src, archive$System[[exp.name]]$src$fcst,
                                  '$ensemble$', table, '$var$', grid, version,
                                  paste0('$var$_', table, '_*_dcppB-forecast_s', sdates_fcst[i_sdate], '-$ensemble$_', grid, '_$chunk$.nc')))
        } else {
          path_list[[i_sdate]] <- 
            list(path = file.path(archive$src, archive$System[[exp.name]]$src$hcst,
                                  '$ensemble$', table, '$var$', grid, version,
                                  paste0('$var$_', table, '_*_dcppA-hindcast_s', sdates_fcst[i_sdate], '-$ensemble$_', grid, '_$chunk$.nc')))
        }
      }
    }
  }

#  fcst.path <- file.path(archive$src, archive$System[[exp.name]]$src$fcst,
#                         '$ensemble$', table, '$var$', grid, version)
#  fcst.files <- paste0('$var$_', table, '_*_*_s$syear$-$ensemble$_', grid, '_$chunk$.nc')


  # monthly & daily
  if (!multi_path) {
    #NOTE: the adjustment for two cases (multiple files per sdate or not) has been made in hcst
    Start_fcst_arg_list <- Start_hcst_arg_list
    Start_fcst_arg_list[['dat']] <- file.path(fcst.path, fcst.files)
    Start_fcst_arg_list[['syear']] <- paste0(sdates_fcst)
    fcst <- do.call(Start, Start_fcst_arg_list)


  } else { # multi_path

    #TODO: time attribute is not correct
    Start_fcst_arg_list <- Start_hcst_arg_list
    Start_fcst_arg_list[['dat']] <- path_list
    Start_fcst_arg_list[['syear']] <- NULL
    Start_fcst_arg_list[['chunk_depends']] <- NULL
    if ('syear' %in% Start_fcst_arg_list[['return_vars']][['time']]) {
      remove_ind <- which(Start_fcst_arg_list[['return_vars']][['time']] == 'syear')
      Start_fcst_arg_list[['return_vars']][['time']] <- Start_fcst_arg_list[['return_vars']][['time']][-remove_ind]
    }

    fcst <- do.call(Start, Start_fcst_arg_list)

    # Reshape and reorder dimensions
    ## dat should be 1, syear should be length of dat; reorder dimensions
    dim(fcst) <- c(dat = 1, syear = as.numeric(dim(fcst))[1], dim(fcst)[2:6])
    fcst <- s2dv::Reorder(fcst, c('dat', 'var', 'ensemble', 'syear', 'time', 'latitude', 'longitude'))

  }

  tmp_time_attr <- attr(fcst, 'Variables')$common$time

  # change syear to c(sday, sweek, syear)
  dim(fcst) <- c(dim(fcst)[1:3], sday = 1, sweek = 1, dim(fcst)[4:7])
  tmp_time_attr <- attr(fcst, 'Variables')$common$time
  #TODO: This check cannot pass if multi_path
  if (!multi_path) {
    if (!identical(dim(tmp_time_attr), dim(fcst)[6:7])) {
      stop("fcst has problem in matching data and time attr dimension.")
    }
    dim(attr(fcst, 'Variables')$common$time) <- c(sday = 1, sweek = 1, dim(fcst)[6:7])
  }

  # Change class from startR_array to s2dv_cube
  suppressWarnings(
  fcst <- as.s2dv_cube(fcst)
  )

  if (!identical(dim(hcst$data)[-6], dim(fcst$data)[-6])) {
    stop("hcst and fcst do not share the same dimension structure.")
  }

  } else {
    fcst <- NULL
  }

#-------------------------------------------
# Step 3. Load the reference
#-------------------------------------------
  obs.path <- file.path(archive$src, archive$Reference[[ref.name]]$src, store.freq,
                        paste0(variable, archive$Reference[[ref.name]][[store.freq]][[variable]]))
  obs.files <- paste0('$var$_$file_date$.nc')

  # Get from startR_cube
#  dates <- attr(hcst, 'Variables')$common$time
  # Get from s2dv_cube
  dates <- hcst$Dates$start 
  dates_file <- sapply(dates, format, '%Y%m')
  dim(dates_file) <- dim(dates)

  if (store.freq == "daily_mean") {
#////////////////
# Method 1: use hcst time attr as obs time selector
#////////////////

  # Set hour to 12:00 to ensure correct date retrieval for daily data
  lubridate::hour(dates) <- 12 
  lubridate::minute(dates) <- 00
  # Restore correct dimensions
  dim(dates) <- dim(dates_file)

  obs <- Start(dat = file.path(obs.path, obs.files),
               var = variable,
               file_date = unique(format(dates, '%Y%m')),
               time = dates, # [sday, sweek, syear, time]
               time_across = 'file_date',
               merge_across_dims = TRUE,
               split_multiselected_dims = TRUE,
               latitude = values(list(lats.min, lats.max)),
               latitude_reorder = Sort(decreasing = TRUE), 
               longitude = values(list(lons.min, lons.max)),
               longitude_reorder = circularsort,
               transform = regrid_params$obs.transform,
               transform_extra_cells = 2,
               transform_params = list(grid = regrid_params$obs.gridtype, #nc file
                                       method = regrid_params$obs.gridmethod),
               transform_vars = c('latitude', 'longitude'),
               synonims = list(latitude = c('lat','latitude'), 
                               longitude = c('lon','longitude')),
               return_vars = list(latitude = NULL, longitude = NULL,
                                  time = 'file_date'),
               retrieve = TRUE)

  } else if (store.freq == "monthly_mean") {
#////////////////
# Method 2: reshape hcst time attr's date into an array with time dim then as obs date selector
#////////////////

  obs <- Start(dat = file.path(obs.path, obs.files),
               var = variable,
               file_date = dates_file, #dates_arr, # [sday, sweek, syear, time]
               split_multiselected_dims = TRUE,
               latitude = values(list(lats.min, lats.max)),
               latitude_reorder = Sort(decreasing = TRUE),
               longitude = values(list(lons.min, lons.max)),
               longitude_reorder = circularsort,
               transform = regrid_params$obs.transform,
               transform_extra_cells = 2,
               transform_params = list(grid = regrid_params$obs.gridtype, #nc file
                                       method = regrid_params$obs.gridmethod),
               transform_vars = c('latitude', 'longitude'),
               synonims = list(latitude = c('lat','latitude'),
                               longitude = c('lon','longitude')),
               return_vars = list(latitude = NULL, longitude = NULL,
                                  time = 'file_date'),
               retrieve = TRUE)
  }


#dim(attr(obs, 'Variables')$common$time)
# sday sweek syear  time 
#    1     1     2    14 


#  # TODO: Reorder obs dims to match hcst dims?
#  # Adds ensemble dim to obs (for consistency with hcst/fcst)
#  default_dims <- c(dat = 1, var = 1, sweek = 1, 
#                    sday = 1, syear = 1, time = 1, 
#                    latitude = 1, longitude = 1, ensemble = 1)
#  default_dims[names(dim(obs))] <- dim(obs)
#  dim(obs) <- default_dims

  if (!identical(dim(obs), dim(hcst$data)[-3])) {
    stop("obs and hcst dimensions do not match.")
  }
  # Add ensemble dim to obs
  dim(obs) <- c(dim(obs)[1:2], ensemble = 1, dim(obs)[3:8])

  # Change class from startR_array to s2dv_cube
  suppressWarnings(
  obs <- as.s2dv_cube(obs)
  )


#-------------------------------------------
# Step 4. Print the summary of data
#-------------------------------------------

  # Print a summary of the loaded data for the user, for each object
  data_summary(hcst, store.freq)
  data_summary(obs, store.freq)
  if (!is.null(fcst)) {
    data_summary(fcst, store.freq)
  }


#-------------------------------------------
# Step 5. Verify the consistance btwn hcst and obs
#-------------------------------------------
  # dimension
  if (any(!names(dim(obs$data)) %in% names(dim(hcst$data)))) {
    stop("hcst and obs don't share the same dimension names.")
  } else {
    ens_ind <- which(names(dim(obs$data)) == 'ensemble')
    match_ind <- match(names(dim(obs$data))[-ens_ind], names(dim(hcst$data)))
    if (!all(dim(hcst$data)[match_ind] == dim(obs$data)[-ens_ind])) stop("hcst and obs don't share the same dimension length.")
  }

  # time attribute
  if (!identical(format(hcst$Dates$start, '%Y%m'),
                 format(obs$Dates$start, '%Y%m')))
    stop("hcst and obs don't share the same time.")

  # lat and lon attributes
  if (!identical(as.vector(hcst$lat),
                 as.vector(obs$lat)))
    stop("hcst and obs don't share the same latitude.")
  if (!identical(as.vector(hcst$lon),
                 as.vector(obs$lon)))
    stop("hcst and obs don't share the same longitude.")

  # Check fcst
  if (!is.null(fcst)) {
    # dimension
    if (any(!names(dim(fcst$data)) %in% names(dim(hcst$data)))) {
      stop("hcst and fcst don't share the same dimension names.")
    } else {
      ens_ind <- which(names(dim(fcst$data)) %in% c('ensemble', 'syear'))
      match_ind <- match(names(dim(fcst$data))[-ens_ind], names(dim(hcst$data)))
    if (!all(dim(hcst$data)[match_ind] == dim(fcst$data)[-ens_ind])) 
      stop("hcst and fcst don't share the same dimension length.")
    }

    # lat and lon attributes
    if (!identical(as.vector(hcst$lat),
                   as.vector(fcst$lat)))
      stop("hcst and fcst don't share the same latitude.")
    if (!identical(as.vector(hcst$lon),
                   as.vector(fcst$lon)))
      stop("hcst and fcst don't share the same longitude.")
  }

  return(list(hcst = hcst, fcst = fcst, obs = obs))
}
