
recipe_file <- "modules/Loading/testing_recipes/recipe_3.yml"

source("modules/Loading/Loading.R")
source("modules/Calibration/Calibration.R")
source("modules/Skill/Skill.R")
source("modules/Saving/Saving.R")

# Load datasets
data <- load_datasets(recipe_file)
## TODO: Instead of reading the recipe at each module, do it at the beginning?
recipe <- read_yaml(recipe_file)
# Calibrate datasets
calibrated_data <- calibrate_datasets(data, recipe)
# Compute skill metrics
skill_metrics <- compute_skill_metrics(calibrated_data$hcst, data$obs,
				       recipe, na.rm = T, ncores = 4)

probs <- compute_probabilities(calibrated_data$hcst, recipe,
				       na.rm = T, ncores = 4)

# Export skill metrics onto outfile
outdir <- get_dir(recipe)
dir.create(outdir, showWarnings = FALSE, recursive = TRUE)
save_metrics(skill_metrics, recipe, data$hcst, outdir)
# Export percentiles and probability bins onto outfile
save_percentiles(probs$percentiles, recipe, data$hcst, outdir)
save_probabilities(probs$probs, recipe, data$hcst, outdir)
# Export hindcast, forecast and observations onto outfile
save_forecast(calibrated_data$hcst, recipe, outdir)
if (!is.null(calibrated_data$fcst)) {
  save_forecast(calibrated_data$fcst, recipe, outdir)
}
save_observations(data$obs, recipe, outdir)
