context("Decadal monthly data - 1")

###########################################

source("modules/Loading/Loading_decadal.R")
source("modules/Calibration/Calibration.R")
source("modules/Skill/Skill.R")
source("modules/Saving/Saving.R")

recipe_file <- "tests/recipes/recipe-decadal_monthly_1.yml"

# Load datasets
suppressWarnings({invisible(capture.output(
data <- load_datasets(recipe_file)
))})

recipe <- read_yaml(recipe_file)
# Calibrate datasets
suppressWarnings({invisible(capture.output(
  calibrated_data <- calibrate_datasets(data, recipe)
))})

# Compute skill metrics
suppressWarnings({invisible(capture.output(
skill_metrics <- compute_skill_metrics(calibrated_data$hcst, data$obs,
                                       recipe, na.rm = T, ncores = 4)
))})
suppressWarnings({invisible(capture.output(
probs <- compute_probabilities(calibrated_data$hcst, recipe,
                                       na.rm = T, ncores = 4)
))})

#======================================

test_that("1. Loading", {

expect_equal(
is.list(data),
TRUE
)
expect_equal(
names(data),
c("hcst", "fcst", "obs")
)
expect_equal(
class(data$hcst),
"s2dv_cube"
)
expect_equal(
class(data$fcst),
"s2dv_cube"
)
expect_equal(
class(data$obs),
"s2dv_cube"
)
expect_equal(
names(data$hcst),
c("data", "Variable", "Datasets", "Dates", "when", "source_files", "load_parameters")
)
expect_equal(
names(data$hcst),
names(data$fcst)
)
expect_equal(
names(data$hcst),
names(data$obs)
)
expect_equal(
dim(data$hcst$data),
c(dat = 1, var = 1, ensemble = 2, sday = 1, sweek = 1, syear = 4, time = 3, latitude = 5, longitude = 4)
)
expect_equal(
dim(data$fcst$data),
c(dat = 1, var = 1, ensemble = 2, sday = 1, sweek = 1, syear = 1, time = 3, latitude = 5, longitude = 4)
)
expect_equal(
dim(data$hcst$Dates$start),
c(sday = 1, sweek = 1, syear = 4, time = 3)
)
expect_equal(
as.vector(drop(data$hcst$data)[,1:2,1,2,3]),
c(291.3831, 291.6227, 292.3012, 290.9779),
tolerance = 0.0001
)
expect_equal(
mean(data$hcst$data),
287.3804,
tolerance = 0.0001
)
expect_equal(
range(data$hcst$data),
c(281.7395, 294.2467),
tolerance = 0.0001
)
expect_equal(
(data$hcst$Dates$start)[1],
as.POSIXct("1991-11-16", tz = 'UTC')
)
expect_equal(
(data$hcst$Dates$start)[2],
as.POSIXct("1992-11-16", tz = 'UTC')
)
expect_equal(
(data$hcst$Dates$start)[5],
as.POSIXct("1991-12-16 12:00:00", tz = 'UTC')
)
expect_equal(
(data$hcst$Dates$start)[10],
as.POSIXct("1993-01-16 12:00:00", tz = 'UTC')
)

})

#======================================
test_that("2. Calibration", {

expect_equal(
is.list(calibrated_data),
TRUE
)
expect_equal(
names(calibrated_data),
c("hcst", "fcst")
)
expect_equal(
class(calibrated_data$hcst),
"s2dv_cube"
)
expect_equal(
class(calibrated_data$fcst),
"s2dv_cube"
)
expect_equal(
dim(calibrated_data$hcst$data),
c(dat = 1, var = 1, ensemble = 2, sday = 1, sweek = 1, syear = 4, time = 3, latitude = 5, longitude = 4)
)
expect_equal(
dim(calibrated_data$fcst$data),
c(dat = 1, var = 1, ensemble = 2, sday = 1, sweek = 1, syear = 1, time = 3, latitude = 5, longitude = 4)
)
expect_equal(
mean(calibrated_data$fcst$data),
291.8375,
tolerance = 0.0001
)
expect_equal(
mean(calibrated_data$hcst$data),
289.6679,
tolerance = 0.0001
)
expect_equal(
as.vector(drop(calibrated_data$hcst$data)[1, , 2, 3, 4]),
c(286.3895, 286.6408, 290.6652, 288.3759),
tolerance = 0.0001
)
expect_equal(
range(calibrated_data$fcst$data),
c(287.2173, 297.4578),
tolerance = 0.0001
)

})


#======================================
test_that("3. Metrics", {

expect_equal(
is.list(skill_metrics),
TRUE
)
expect_equal(
names(skill_metrics),
c("rpss", "rpss_significance")
)
expect_equal(
class(skill_metrics$rpss),
"array"
)
expect_equal(
dim(skill_metrics$rpss),
c(time = 3, latitude = 5, longitude = 4)
)
expect_equal(
dim(skill_metrics$rpss_significance),
dim(skill_metrics$rpss)
)
expect_equal(
as.vector(drop(skill_metrics$rpss)[, 2, 3]),
c(-0.2857143, -1.2500000, -1.8928571),
tolerance = 0.0001
)
expect_equal(
as.vector(drop(skill_metrics$rpss_significance)[, 2, 3]),
rep(FALSE, 3)
)

# Probs
expect_equal(
names(probs),
c('probs', 'percentiles')
)
expect_equal(
names(probs$probs),
c('prob_b33', 'prob_33_to_66', 'prob_a66', 'prob_b10', 'prob_10_to_90', 'prob_a90')
)
expect_equal(
names(probs$percentiles),
c('percentile_33', 'percentile_66', 'percentile_10', 'percentile_90')
)
expect_equal(
dim(probs$probs$prob_b33),
c(syear = 4, time = 3, latitude = 5, longitude = 4)
)
expect_equal(
dim(probs$percentiles$percentile_33),
c(time = 3, latitude = 5, longitude = 4)
)
expect_equal(
as.vector(probs$probs$prob_b33[, 1, 2, 2]),
c(0.0, 0.5, 0.0, 1.0)
)
expect_equal(
as.vector(probs$probs$prob_10_to_90[, 1, 2, 2]),
c(1.0, 1.0, 0.5, 0.5)
)
expect_equal(
as.vector(probs$percentiles$percentile_33[, 1, 2]),
c(293.7496, 287.4263, 285.8295),
tolerance = 0.0001
)
expect_equal(
as.vector(probs$percentiles$percentile_10[, 1, 2]),
c(293.1772, 286.9533, 284.7887),
tolerance = 0.0001
)

})


